#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.gallery.interfaces import IGallery, IGalleryFile, IGalleryTarget
from pyams_content.component.gallery.zmi.interfaces import IGalleryImageAddFields, IGalleryImagesView
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_file.interfaces.archive import IArchiveExtractor
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager, IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, VIEW_PERMISSION
from z3c.form.interfaces import NOT_CHANGED
from zope.schema.interfaces import WrongType

# import packages
from pyams_content.component.gallery.file import GalleryFile
from pyams_content.shared.common.zmi import WfSharedContentPermissionMixin
from pyams_file.file import get_magic_content_type
from pyams_file.zmi.file import FilePropertiesAction
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.group import NamedWidgetsGroup
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarAction, JsToolbarActionItem
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm, AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field
from zope.lifecycleevent import ObjectCreatedEvent

from pyams_content import _


@viewlet_config(name='add-image.menu', context=IGallery, view=IGalleryImagesView, manager=IWidgetTitleViewletManager)
class GalleryImageAddMenu(WfSharedContentPermissionMixin, ToolbarAction):
    """Gallery image add menu"""

    label = _("Add image(s)")

    url = 'add-image.html'
    modal_target = True
    stop_propagation = True


@pagelet_config(name='add-image.html', context=IGallery, layer=IPyAMSLayer, permission=MANAGE_CONTENT_PERMISSION)
class GalleryImageAddForm(AdminDialogAddForm):
    """Gallery image add form"""

    legend = _("Add image(s)")
    icon_css_class = 'fa -fa-fw fa-picture-o'

    fields = field.Fields(IGalleryImageAddFields)
    ajax_handler = 'add-image.json'

    def updateWidgets(self, prefix=None):
        super(GalleryImageAddForm, self).updateWidgets(prefix)
        if 'author_comments' in self.widgets:
            self.widgets['author_comments'].widget_css_class = 'textarea'

    def create(self, data):
        medias = []
        images = data['images_data']
        if images and (images is not NOT_CHANGED):
            filename = None
            if isinstance(images, (list, tuple)):
                filename, images = images
            if hasattr(images, 'seek'):
                images.seek(0)
            registry = self.request.registry
            content_type = get_magic_content_type(images)
            if isinstance(content_type, bytes):
                content_type = content_type.decode()
            if hasattr(images, 'seek'):
                images.seek(0)
            extractor = query_utility(IArchiveExtractor, name=content_type)
            if extractor is not None:
                extractor.initialize(images)
                for content, filename in extractor.get_contents():
                    try:
                        media = GalleryFile()
                        media.data = filename, content
                    except WrongType:
                        continue
                    else:
                        registry.notify(ObjectCreatedEvent(media))
                        medias.append(media)
            else:
                try:
                    media = GalleryFile()
                    media.data = filename, images if filename else images
                except WrongType:
                    pass
                else:
                    registry.notify(ObjectCreatedEvent(media))
                    medias.append(media)
            for media in medias:
                media.author = data.get('author')
                media.author_comments = data.get('author_comments')
                self.context.append(media)
        return None


@view_config(name='add-image.json', context=IGallery, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class GalleryImageAJAXAddForm(AJAXAddForm, GalleryImageAddForm):
    """Gallery image add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': absolute_url(self.context, self.request, 'get-gallery-images.html'),
                'target': '#gallery_images_{0}'.format(self.context.__name__)}


@viewlet_config(name='file.showhide.action', context=IGalleryFile, layer=IPyAMSLayer, view=IGalleryImagesView,
                manager=IContextActions, permission=VIEW_SYSTEM_PERMISSION, weight=1)
class GalleryFileShowAddAction(JsToolbarActionItem):
    """Gallery file show/hide action"""

    label = _("Show/hide image")

    @property
    def label_css_class(self):
        if self.context.visible:
            return 'fa fa-fw fa-eye'
        else:
            return 'fa fa-fw fa-eye-slash text-danger'

    hint_gravity = 'nw'

    url = 'PyAMS_content.galleries.switchImageVisibility'


@viewlet_config(name='file.properties.action', context=IGalleryFile, layer=IPyAMSLayer, view=IGalleryImagesView,
                manager=IContextActions, permission=VIEW_SYSTEM_PERMISSION, weight=5)
class GalleryFilePropertiesAction(FilePropertiesAction):
    """Media properties action"""

    url = 'gallery-file-properties.html'


@pagelet_config(name='gallery-file-properties.html', context=IGalleryFile, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class GalleryFilePropertiesEditForm(AdminDialogEditForm):
    """Gallery file properties edit form"""

    legend = _("Update image properties")
    icon_css_class = 'fa fa-fw fa-picture-o'
    dialog_class = 'modal-large'

    fields = field.Fields(IGalleryFile).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'gallery-file-properties.json'

    @property
    def title(self):
        gallery = get_parent(self.context, IGallery)
        return II18n(gallery).query_attribute('title', request=self.request)

    def updateWidgets(self, prefix=None):
        super(GalleryFilePropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'author_comments' in self.widgets:
            self.widgets['author_comments'].widget_css_class = 'textarea'
        if 'pif_number' in self.widgets:
            self.widgets['pif_number'].input_css_class = 'col-md-3'
        if 'sound_description' in self.widgets:
            self.widgets['sound_description'].widget_css_class = 'textarea'
        self.add_group(NamedWidgetsGroup(self, 'audio_file', self.widgets,
                                         ('sound', 'sound_title', 'sound_description'),
                                         bordered=False,
                                         legend=_("Audio content"),
                                         css_class='inner',
                                         switch=True,
                                         hide_if_empty=True))


@view_config(name='gallery-file-properties.json', context=IGalleryFile, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class GalleryFileInfoPropertiesAJAXEditForm(AJAXEditForm, GalleryFilePropertiesEditForm):
    """Gallery file properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(GalleryFileInfoPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'title' in changes.get(IGalleryFile, ()):
            gallery = get_parent(self.context, IGallery)
            if gallery is not None:
                output.setdefault('events', []).append({
                    'event': 'PyAMS_content.changed_item',
                    'options': {
                        'handler': 'PyAMS_content.galleries.updateImageTitle',
                        'image_id': 'image_{0}_{1}'.format(gallery.__name__, self.context.__name__),
                        'title': II18n(self.context).query_attribute('title', request=self.request)
                    }
                })
        return output


@viewlet_config(name='gallery-file-download.action', context=IGalleryFile, layer=IPyAMSLayer, view=IGalleryImagesView,
                manager=IContextActions, permission=VIEW_PERMISSION, weight=89)
class GalleryFileDownloaderAction(JsToolbarActionItem):
    """Gallery file downloader action"""

    label = _("Download image...")
    label_css_class = 'fa fa-fw fa-download'
    hint_gravity = 'nw'

    @property
    def url(self):
        return absolute_url(self.context.data, self.request, query={'download': '1'})


@viewlet_config(name='gallery-file-remover.action', context=IGalleryFile, layer=IPyAMSLayer, view=IGalleryImagesView,
                manager=IContextActions, weight=90)
class GalleryFileRemoverAction(WfSharedContentPermissionMixin, JsToolbarActionItem):
    """Gallery file remover action"""

    label = _("Remove image...")
    label_css_class = 'fa fa-fw fa-trash'
    hint_gravity = 'nw'

    url = 'PyAMS_content.galleries.removeFile'


@view_config(name='delete-element.json', context=IGallery, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def delete_gallery_element(request):
    """Delete gallery element"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("No provided object_name argument!"))}}
    if name not in request.context:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("Given image name doesn't exist!"))}}
    del request.context[name]
    return {'status': 'success'}
